# Generated by Django 4.2.5 on 2023-11-06 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reporting_endpoints', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='report',
            name='disposition',
            field=models.CharField(choices=[('enforce', 'Enforce'), ('report', 'Report'), ('unknown', 'Unknown')], max_length=128),
        ),
    ]
