# Generated by Django 4.2.5 on 2023-09-27 08:11
from __future__ import annotations

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Report",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "endpoint_name",
                    models.CharField(blank=True, max_length=1024, null=True),
                ),
                ("age", models.PositiveIntegerField(blank=True, null=True)),
                ("type", models.CharField(blank=True, max_length=256, null=True)),
                ("url", models.CharField(blank=True, max_length=1024, null=True)),
                (
                    "user_agent",
                    models.CharField(blank=True, max_length=1024, null=True),
                ),
                ("blocked_uri", models.CharField(max_length=1024)),
                (
                    "disposition",
                    models.CharField(
                        choices=[("enforce", "Enforce"), ("report", "Report")],
                        max_length=128,
                    ),
                ),
                ("document_uri", models.CharField(max_length=1024)),
                ("effective_directive", models.CharField(max_length=1024)),
                ("original_policy", models.TextField()),
                ("referrer", models.CharField(max_length=1024)),
                ("status_code", models.PositiveIntegerField()),
                ("violated_directive", models.CharField(max_length=1024)),
                ("sample", models.CharField(max_length=1024)),
                (
                    "source_file",
                    models.CharField(blank=True, max_length=1024, null=True),
                ),
                ("line_number", models.PositiveIntegerField(blank=True, null=True)),
                ("column_number", models.PositiveIntegerField(blank=True, null=True)),
            ],
        ),
    ]
